@@ -70,7 +70,7 @@
 #'}
 #' # Build the co-association matrix
 #'
-#'C <- matrix(1, N,N)
+#'C <- matrix(NA, N,N)
 #'for (i in 1:(N-1)){
 #'  for (j in (i+1):N){
 #'    C[i,j] <- sum(a[,i]==a[,j])/H
@@ -125,34 +125,19 @@ Cg  <- matrix(NA, ncol=3, nrow=k)
       ind.gi  <- rbind(ind.gi,c(j,sort(C[j,com.gi],
         decreasing=TRUE)))
       ind.gi1 <- rbind(ind.gi1,c(j,
-        # criteria involving maximization
-        #max(C[j,com.gi],na.rm=TRUE),          #dropped
         sum(C[j,com.gi],na.rm=TRUE),           #maxsumint
-        # criteria involving minimization
-        #min(C[j,com.gi],na.rm=TRUE),          #dropped
-        #min(C[j,com.ngi]),                    #dropped
         sum(C[j,com.ngi]),                     #minsumnoint
-        # another criterion involving maximization
         sum(C[j,com.gi],na.rm=TRUE)-sum(C[j,com.ngi])))
   }                                            #maxsumdiff
 
 # Methods: from 1 to 4
-    # if (!is.null(ind.gi))
-    #   Cg[g.i, 1] <- com.gi[which.max(ind.gi1[,2])]
+
     if (!is.null(ind.gi))
       Cg[g.i, 1] <- com.gi[which.max(ind.gi1[,2])]
     if (!is.null(ind.gi))
       Cg[g.i, 2] <- com.gi[which.min(ind.gi1[,3])]
     if (!is.null(ind.gi))
-      Cg[g.i, 3] <- com.gi[which.min(ind.gi1[,4])]
-    # if (!is.null(ind.gi))
-    #   Cg[g.i, 5] <- com.gi[which.min(ind.gi1[,6])]
-    # if (!is.null(ind.gi))
-    #   Cg[g.i, 6] <- com.gi[which.max(ind.gi1[,7])]
-    # if (available_met==4){
-    # if (!is.null(ind.gi))
-    #   Cg[, 4]    <-  t(maxima)
-    # }
+      Cg[g.i, 3] <- com.gi[which.max(ind.gi1[,4])]
     if (!is.null(ind.gi)){
       Cg1[g.i] <- ind.gi[do.call(order,
         as.data.frame(-ind.gi[,-1]))[1],1]
@@ -163,35 +148,7 @@ Cg  <- matrix(NA, ncol=3, nrow=k)
 Cg <- Cg[,1:3]
 colnames(Cg) <- c("maxsumint", "minsumnoint", "maxsumdiff")
 
-# group1 contains the observation assigments to the groups obtained via pivots
-# group1 <- 0*Z
-# # cycle on iterations
-#   for (i in 1:ncol(Z)){
-# # cycle on number of groups
-#     for (j in 1:k){
-#       if (!is.na(Cg[j])){
-#         group1[Z[,i] ==Z[Cg[j],i],i] <- j
-#       }
-#     }
-#   }
-#
-# # definition of the probabilities to belong to the groups for each unit
-# pr <- matrix(NA,nrow=k,ncol=N)
-#  for (kk in 1:k){
-#   pr[kk,] <- apply(group1,1,FUN=function(x) sum(x==kk)/length(x))
-#  }
-#
-# # definition of the submatrix corresponding to the pivotal units
-#
-# submatrix <- round(C[Cg,Cg],5)
-# T <- max(submatrix[upper.tri(submatrix)])
-
-  return(list(
-    #pr=pr,
-     pivots=Cg
-    #Submatrix=submatrix,
-    #Max=T
-    ))
+  return(list(pivots=Cg))
 }
 
 
